%% Description
% This script calculates the aeroelastic stability limit based on 
% the general eigenvalue problem where several modes is chosen to described
% the problem. The script import the data from the *.dat file from ABAQUS
% by the function read_ABAQUS_dat.m created by Ole Andre Oiseth, and 
% slightly modified by the author to include cabledisplacements. 
% The function aeroelasticstab.m is used to perform the iteration process. 
% Date: 10.06.2012  Haavard Maurset
clc
clear all 
close all 
%%
tic
VALGT=[];           % Number of modes to be included
Nodes=185;          % Number of nodes along the girder where displacements 
                    % are extracted
CNodes=187;         % Nodes along the cable for extracting displacements
L=3700;             % Length of girder
%% Importing the data from ABAQUS
[Eigenvectors,Eigenvalues,CableEigenvec]...
    =read_ABAQUS_dat('filename.dat',Nodes,CNodes);
omegaomega=(Eigenvalues(VALGT,3));
Modalmass=Eigenvalues(VALGT,5);

%% Defining the modes and sort the table by the x-coordinate (along bridge)
Coordinates=Eigenvectors(:,2:4);
[~, b]=sort(Coordinates(:,1));

%% Calculating the modeintegrals
% Importing the modes from ABAQUS. The first component of a mode is the 
% horizontal(lateral), second vertical and the third torsional 
% displacement

phiphi=zeros(3,size(Eigenvectors,1),length(omegaomega));
teller=1;%min(VALGT);
for n=1:length(omegaomega)
    phiphi(2,:,teller)=Eigenvectors(b,6,VALGT(n))';%n+min(VALGT)-1)';
    phiphi(1,:,teller)=Eigenvectors(b,7,VALGT(n))';%n+min(VALGT)-1)';
    phiphi(3,:,teller)=Eigenvectors(b,8,VALGT(n))';%n+min(VALGT)-1)';
    teller=teller+1;
end

moderInt=zeros(3,3,size(phiphi,3),size(phiphi,3));
for i=1:size(phiphi,3)
    for j=1:size(phiphi,3)
        for n=1:3
            for m=1:3
                integrand(1,:)=phiphi(n,:,i).*phiphi(m,:,j);
                moderInt(n,m,i,j)=trapz(integrand)*L/Nodes;                    
            end
        end       
    end
end

%% Structural properties

M=diag(Modalmass);                          % Modal mass matrix

% Natural frequencies and damping ratios
omega0=diag(omegaomega);
zeta=diag(0.05*ones(1,length(omegaomega)));
% Modal matrices
K=omega0.^2*M;                              % Modal stiffness matrix
C=2.*zeta.*M*omega0;                        % Modal damping matrix
% Constants
ro=1.25;                                    % Density of air
B=7.5;                                      % Width of one girder


%% Eigenvalue solution
maxfit=200;                 % Max number of frequency and velocity 
maxVit=1000;                % iterations

V0=0;                       % Initial mean wind velocity
dV=1;                       % Velocity increment
omg0=1;                     % Initial assumption of critical frequency

[Omega,v_n,omega_n,omega,Vvec,ImLambda,ReLambda,Vcr,OmegaCr]=aeroelasticstab(@aerodynamicderivativesBS15GV,M,C,K,omg0,moderInt,ro,B,maxVit,maxfit,V0,dV,phiphi);



%% Results
figure(1)
set(gcf,'Name','Solution of the eigenvalue problem','Numbertitle','off')
plot(subplot(2,1,1),Vvec,ImLambda,'o')
ylabel('Im($\lambda_n$)','interpreter','latex')
title('Imaginary part of the eigenvalues (damped natural frequencies)','interpreter','latex')

plot(subplot(2,1,2),Vvec,ReLambda,'o'),grid on
xlabel('V (m/s)','interpreter','latex')
ylabel('Re($\lambda_n$)','interpreter','latex')
title('Real part of the eigenvalues (damping term,$\mu_n= -v_{n}\omega_n$)','interpreter','latex')

disp(['The aeroelastic stability limit is ', num2str(Vcr), ' m/s'])
disp(['The critical frequency is ' ,num2str(OmegaCr), ' rad/s'])

CPUtime=toc;



