%% Description
% This script calculates the aeroelastic stability limit based on 
% the general eigenvalue problem where several modes is chosen to described
% the problem. The script import the data from the *.dat file from ABAQUS
% by the function read_ABAQUS_dat.m created by Ole Andre Oiseth, and 
% slightly modified by the author to include Cable displacements. The 
% function aeroestab_Havard.m is used to perform the iteration process. 
% Date: 10.06.2012  Haavard Maurset
clc
clear all 
close all 
%% 
tic
VALGT=[];           % Modes to be included in the calculations
Nodes=185;          % Number of nodes along the girder where displacements 
                    % are extracted
CNodes=187;         % Nodes along the cable for extracting displacements
L=3700;             % Length of girder
%% Importing the data from ABAQUS
[Eigenvectors,Eigenvalues,CableEigenvec]...
    =read_ABAQUS_dat('filename.dat',Nodes,CNodes);
omegaomega=(Eigenvalues(VALGT,3));
Modalmass=Eigenvalues(VALGT,5);

%% Defining the modes and sort the table by the x-coordinate (along bridge)
Coordinates=Eigenvectors(:,2:4);
[~, b]=sort(Coordinates(:,1));

%% Calculating the modeintegrals
% Importing the modes from ABAQUS. The first component of a mode is the 
% vertical, second horizontal(lateral) and the third torsional 
% displacements

phiphi=zeros(3,size(Eigenvectors,1),length(omegaomega));
teller=1;%min(VALGT);
for n=1:length(omegaomega)
    phiphi(2,:,teller)=Eigenvectors(b,6,VALGT(n))';%n+min(VALGT)-1)';
    phiphi(1,:,teller)=Eigenvectors(b,7,VALGT(n))';%n+min(VALGT)-1)';
    phiphi(3,:,teller)=Eigenvectors(b,8,VALGT(n))';%n+min(VALGT)-1)';
    teller=teller+1;
end

moderInt=zeros(3,3,size(phiphi,3),size(phiphi,3));
for i=1:size(phiphi,3)
    for j=1:size(phiphi,3)
        for n=1:3
            for m=1:3
                integrand(1,:)=phiphi(n,:,i).*phiphi(m,:,j);
                moderInt(n,m,i,j)=trapz(integrand)*L/Nodes;                    
            end
        end       
    end
end

%% Structural properties
M=diag(Modalmass);                              % Modal mass matrix

% Natural frequencies and damping ratios
omega=omegaomega;
zeta=diag(0.05*ones(1,length(omegaomega)));
% Modal matrices
K=diag(omega).^2*M;                             % Modal stiffness matrix
C=2.*zeta.*M*diag(omega);                       % Modal damping matrix
% Constants
ro=1.25;                                        % Density of air
B=7.5;                                          % Width of one girder


%% Eigenvalue solution
maxfit=200;                 % Max number of frequency and velocity 
maxVit=1000;                % iterations

V=1;                        % Initial mean wind velocity
dV=1;                       % Velocity increment

A=[];
[Vvec,imS,reS,VCR,OmegaCR]=aerostab_Havard(@aerodynamicderivativesBS15GV,M,C,K,omega,moderInt,ro,B,maxfit,maxVit,V,dV,A);



%% Results
figure(1)
set(gcf,'Name','Solution of the eigenvalue problem','Numbertitle','off')
plot(subplot(2,1,1),Vvec,imS,'o')
ylabel('$Im(\lambda_n)$','interpreter','latex')
title('Imaginary part of the eigenvalues (damped natural frequencies)',...
    'interpreter','latex')

plot(subplot(2,1,2),Vvec,reS,'o'),grid on
xlabel('V (m/s)','interpreter','latex')
ylabel('$Re(\lambda_n)$','interpreter','latex')
title('Real part of the eigenvalues (damping term,$\mu_n= -v_{n}\omega_n$)','interpreter','latex')

disp(['The aeroelastic stability limit is ', num2str(VCR), ' m/s'])
disp(['The critical frequency is ' ,num2str(OmegaCR), ' rad/s'])

CPUtime=toc;
disp(['Computationaltime:',num2str(floor(CPUtime/60)),'min',...
    num2str(floor((CPUtime/60-floor(CPUtime/60))*60)),'sek'])


